using DG.Tweening;
using FL.Battle.Components.Actions;
using System;
using UnityEngine;
using XGame.Framework;
using XGame.Framework.Components;
using XGame.Framework.Map;

namespace FL.Battle.Components
{
    public interface IMoveContext
    {
        /// <summary>
        /// ߵĸڵ
        /// </summary>
        Transform OwnerTr { get; }
    }
    public enum EMoveType
    {
        /// <summary>
        /// ֱ˶
        /// DoTween.DOMove
        /// </summary>
        Line = 0,
        /// <summary>
        /// ߹켣
        /// </summary>
        Bezier,
        /// <summary>
        /// 
        /// </summary>
        Parabola,
    }
    /// <summary>
    /// ƶ
    /// </summary>
    public struct MoveArgs
    {
        public Vector3 target;
        public float speed;
        public EMoveType moveType;
        public Ease ease;
        /// <summary>
        /// ӳ٣λ: 
        /// </summary>
        public int delay;
        //public Action<Vector3> onUpdate;
        public Action onComplete;
    }
    /// <summary>
    /// DoTween.DOLocalMoveY 
    /// </summary>
    public class MoveComponent : Component<IMoveContext>, IParallelTarget
    {
        private IAction _action;
        public Vector3 MoveDir { get; private set; }
        public bool IsMoving => _action != null;

        Vector3 IParallelTarget.Position => Context.OwnerTr.position;
        public void MoveTo(MoveArgs args)
        {
            var position = Context.OwnerTr.position;
            var duration = Mathf.RoundToInt(Vector3.Distance(args.target, position) * 1000 / args.speed);
            MoveDir = Vector3.Normalize(args.target - position);
            void OnCompleted()
            {
                ObjectPool.Recycle(_action);
                _action = null;
                MoveDir = Vector3.zero;
                args.onComplete.SafeInvoke();
            }
            var actionArgs = new MoveActionArgs()
            {
                from = position,
                to = args.target,
                duration = duration,
                delay = args.delay,
                ease = args.ease,
                //onUpdate = args.onUpdate,
                onComplete = OnCompleted
            };
            
            if (args.moveType == EMoveType.Line)
            {
                var move = ObjectPool.Acquire<MoveLineAction>();
                _action = move;
                move.args = actionArgs;
                move.Start(Context.OwnerTr);
            }
            else if (args.moveType == EMoveType.Bezier)
            {
                var move = ObjectPool.Acquire<MoveBezierAction>();
                _action = move;
                move.args = actionArgs;
                move.Start(Context.OwnerTr, false);
            }
            else
            {
                var move = ObjectPool.Acquire<MoveParabolaAction>();
                _action = move;
                move.args = actionArgs;
                move.Start(Context.OwnerTr);
            }
        }
        public void StopMove()
        {
            if (_action != null )
            {
                ObjectPool.Recycle(_action);
                _action = null;
            }
            MoveDir = Vector3.zero;
        }
        protected override void OnDispose()
        {
            StopMove();
        }
    }
}
